@@ -29,6 +29,22 @@ impl<E: SystemEvent> ActorContext<E> {
         let path = self.path.clone() / name;
         self.system.get_actor(&path).await
     }
+
+    /// Retrieve or create a new child under this actor if it does not exist yet
+    pub async fn get_or_create_child<A, F>(&self, name: &str, actor_fn: F) -> Result<ActorRef<E, A>, ActorError>
+    where
+        A: Actor<E>,
+        F: FnOnce() -> A
+    {
+        let path = self.path.clone() / name;
+        self.system.get_or_create_actor_path(&path, actor_fn).await
+    }
+
+    /// Stops the child actor
+    pub async fn stop_child(&self, name: &str) {
+        let path = self.path.clone() / name;
+        self.system.stop_actor(&path).await;
+    }
 }
 
 /// Defines what an actor will receive as its message, and with what it should respond.
