@@ -2,7 +2,7 @@ use std::{any::Any, collections::HashMap, sync::Arc};
 
 use tokio::sync::RwLock;
 
-use crate::{ActorError, ActorPath, actor::{Actor, ActorRef, runner::ActorRunner}, bus::{EventBus, EventConsumer}};
+use crate::{ActorError, ActorPath, actor::{Actor, ActorRef, runner::ActorRunner}, bus::{EventBus, EventReceiver}};
 
 /// Events that this actor system will send
 pub trait SystemEvent: Clone + Send + Sync + 'static {}
@@ -34,7 +34,7 @@ impl<E: SystemEvent> ActorSystem<E> {
     }
 
     /// Subscribe to events of this actor system.
-    pub fn events(&self) -> EventConsumer<E> {
+    pub fn events(&self) -> EventReceiver<E> {
         self.bus.subscribe()
     }
 
@@ -77,15 +77,40 @@ impl<E: SystemEvent> ActorSystem<E> {
         self.create_actor_path(path, actor).await
     }
 
+    /// Retrieve or create a new actor on this actor system if it does not exist yet.
+    pub async fn get_or_create_actor<A, F>(&self, name: &str, actor_fn: F) -> Result<ActorRef<E, A>, ActorError>
+    where
+        A: Actor<E>,
+        F: FnOnce() -> A
+    {
+        let path = ActorPath::from("/user") / name;
+        self.get_or_create_actor_path(&path, actor_fn).await
+    }
+
+    pub(crate) async fn get_or_create_actor_path<A, F>(&self, path: &ActorPath, actor_fn: F) -> Result<ActorRef<E, A>, ActorError>
+    where
+        A: Actor<E>,
+        F: FnOnce() -> A
+    {
+        let actors = self.actors.read().await;
+        match self.get_actor(path).await {
+            Some(actor) => Ok(actor),
+            None => {
+                drop(actors);
+                self.create_actor_path(path.clone(), actor_fn()).await
+            }
+        }
+    }
+
     /// Stops the actor on this actor system. All its children will also be stopped.
     pub async fn stop_actor(&self, path: &ActorPath) {
         log::debug!("Stopping actor '{}' on system '{}'...", &path, &self.name);
-        let mut paths: Vec<ActorPath> = Vec::new();
+        let mut paths: Vec<ActorPath> = vec![path.clone()];
         {
-            let current_actors = self.actors.read().await;
-            for child in current_actors.keys() {
-                if child.is_descendant_of(path) {
-                    paths.push(child.clone());
+            let running_actors = self.actors.read().await;
+            for running in running_actors.keys() {
+                if running.is_descendant_of(path) {
+                    paths.push(running.clone());
                 }
             }
         }
@@ -164,6 +189,7 @@ mod tests {
     #[async_trait]
     impl Actor<TestEvent> for OtherActor {
         async fn pre_start(&mut self, ctx: &mut ActorContext<TestEvent>) {
+            log::debug!("OtherActor initial message: {}", &self.message);
             let child = TestActor { counter: 0 };
             self.child = ctx.create_child("child", child).await.ok();
         }
@@ -211,6 +237,26 @@ mod tests {
         assert_eq!(result, 1);
     }
 
+    #[tokio::test]
+    async fn actor_get_or_create() {
+        if std::env::var("RUST_LOG").is_err() {
+            std::env::set_var("RUST_LOG", "trace");
+        }
+        let _ = env_logger::builder().is_test(true).try_init();
+
+        let bus = EventBus::<TestEvent>::new(1000);
+        let system = ActorSystem::new("test", bus);
+
+        let initial_message = "hello world!".to_string();
+        let actor_fn = || OtherActor { message: initial_message, child: None };
+        let mut actor_ref = system.get_or_create_actor("test-actor", actor_fn ).await.unwrap();
+
+        let msg = OtherMessage("Updated message.".to_string());
+        let result = actor_ref.ask(msg).await.unwrap();
+
+        assert_eq!(result, "Updated message.".to_string());
+    }
+
     #[tokio::test]
     async fn actor_stop() {
         if std::env::var("RUST_LOG").is_err() {
